# Generated by Django 2.2.12 on 2020-06-13 00:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("fleet", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="fleet",
            name="name",
            field=models.CharField(default="", max_length=50),
        ),
    ]



# Generated by Django 2.2.12 on 2020-06-13 17:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("groupmanagement", "0013_fix_requestlog_date_field"),
        ("fleet", "0002_fleet_name"),
    ]

    operations = [
        migrations.AddField(
            model_name="fleet",
            name="groups",
            field=models.ManyToManyField(
                help_text="Group listed here will be able to join the fleet",
                related_name="restricted_groups",
                to="groupmanagement.AuthGroup",
            ),
        ),
    ]



from .providers import esi
from .models import Fleet, FleetInformation
from esi.models import Token
from celery import shared_task
from django.utils import timezone
from concurrent.futures import ThreadPoolExecutor, as_completed
import logging

logger = logging.getLogger(__name__)


@shared_task
def open_fleet(character_id, motd, free_move, name, groups):
    required_scopes = ["esi-fleets.read_fleet.v1", "esi-fleets.write_fleet.v1"]
    c = esi.client
    token = Token.get_token(character_id, required_scopes)
    fleet_result = c.Fleets.get_characters_character_id_fleet(
        character_id=token.character_id, token=token.valid_access_token()
    ).result()
    fleet_id = fleet_result.pop("fleet_id")
    fleet_role = fleet_result.pop("role")

    if fleet_id == None or fleet_role == None or fleet_role != "fleet_commander":
        return

    fleet = Fleet(
        fleet_id=fleet_id,
        created_at=timezone.now(),
        motd=motd,
        is_free_move=free_move,
        fleet_commander_id=token.character_id,
        name=name,
    )
    fleet.save()
    fleet.groups.set(groups)

    esiFleet = {"is_free_move": free_move, "motd": motd}
    c.Fleets.put_fleets_fleet_id(
        fleet_id=fleet_id, token=token.valid_access_token(), new_settings=esiFleet
    ).result()


@shared_task
def send_fleet_invitation(character_ids, fleet_id):
    required_scopes = ["esi-fleets.write_fleet.v1"]
    c = esi.client
    fleet = Fleet.objects.get(fleet_id=fleet_id)
    fleet_commander_token = Token.get_token(fleet.fleet_commander_id, required_scopes)
    _processes = []
    with ThreadPoolExecutor(max_workers=50) as ex:
        for _chracter_id in character_ids:
            _processes.append(
                ex.submit(
                    send_invitation,
                    character_id=_chracter_id,
                    fleet_commander_token=fleet_commander_token,
                    fleet_id=fleet_id,
                )
            )
    for item in as_completed(_processes):
        _ = item.result()


@shared_task
def send_invitation(character_id, fleet_commander_token, fleet_id):
    c = esi.client
    invitation = {"character_id": character_id, "role": "squad_member"}
    c.Fleets.post_fleets_fleet_id_members(
        fleet_id=fleet_id,
        token=fleet_commander_token.valid_access_token(),
        invitation=invitation,
    ).result()


@shared_task
def check_fleet_adverts():
    required_scopes = ["esi-fleets.read_fleet.v1", "esi-fleets.write_fleet.v1"]
    c = esi.client
    fleets = Fleet.objects.all()
    for fleet in fleets:
        token = Token.get_token(fleet.fleet_commander_id, required_scopes)
        try:
            fleet_result = c.Fleets.get_characters_character_id_fleet(
                character_id=token.character_id, token=token.valid_access_token()
            ).result()
            fleet_id = fleet_result["fleet_id"]
            if fleet_id != fleet.fleet_id:
                fleet.delete()
        except Exception as e:
            if e.status_code == 404:  # 404 means the character is not in a fleet
                fleet.delete()
                logger.info("Character is not in a fleet - fleet advert removed")


@shared_task
def get_fleet_composition(fleet_id):
    required_scopes = ["esi-fleets.read_fleet.v1", "esi-fleets.write_fleet.v1"]
    c = esi.client
    fleet = Fleet.objects.get(fleet_id=fleet_id)
    token = Token.get_token(fleet.fleet_commander_id, required_scopes)
    fleet_infos = c.Fleets.get_fleets_fleet_id_members(
        fleet_id=fleet_id, token=token.valid_access_token()
    ).result()

    characters = {}
    systems = {}
    ship_type = {}

    for member in fleet_infos:
        characters[member["character_id"]] = ""
        systems[member["solar_system_id"]] = ""
        ship_type[member["ship_type_id"]] = ""
    ids = []
    ids.extend(list(characters.keys()))
    ids.extend(list(systems.keys()))
    ids.extend(list(ship_type.keys()))

    ids_to_name = c.Universe.post_universe_names(ids=ids).result()
    for member in fleet_infos:
        index = [x["id"] for x in ids_to_name].index(member["character_id"])
        member["character_name"] = ids_to_name[index]["name"]
    for member in fleet_infos:
        index = [x["id"] for x in ids_to_name].index(member["solar_system_id"])
        member["solar_system_name"] = ids_to_name[index]["name"]
    for member in fleet_infos:
        index = [x["id"] for x in ids_to_name].index(member["ship_type_id"])
        member["ship_type_name"] = ids_to_name[index]["name"]

    aggregate = get_fleet_aggregate(fleet_infos)

    differential = dict()

    for key, value in aggregate.items():
        fleet_info_agg = FleetInformation.objects.filter(
            fleet__fleet_id=fleet_id, ship_type_name=key
        )
        if fleet_info_agg.count() > 0:
            differential[key] = value - fleet_info_agg.latest("date").count
        else:
            differential[key] = value
        FleetInformation.objects.create(fleet=fleet, ship_type_name=key, count=value)

    return FleetViewAggregate(fleet_infos, aggregate, differential)


@shared_task
def get_fleet_aggregate(fleet_infos):
    counts = dict()

    for member in fleet_infos:
        type_ = member.get("ship_type_name")
        if type_ in counts:
            counts[type_] += 1
        else:
            counts[type_] = 1
    return counts


class FleetViewAggregate(object):
    def __init__(self, fleet, aggregate, differential):
        self.fleet = fleet
        self.aggregate = aggregate
        self.differential = differential



from django.utils.translation import ugettext_lazy as _

from allianceauth.services.hooks import MenuItemHook, UrlHook
from allianceauth import hooks

from . import urls


class FleetMenuItem(MenuItemHook):
    """ This class ensures only authorized users will see the menu entry """

    def __init__(self):
        # setup menu entry for sidebar
        MenuItemHook.__init__(
            self,
            _("Fleet"),
            "fa fa-users fa-fw",
            "fleet:dashboard",
            navactive=["fleet:"],
        )

    def render(self, request):
        if request.user.has_perm("fleet.fleet_access"):
            return MenuItemHook.render(self, request)
        return ""


@hooks.register("menu_item_hook")
def register_menu():
    return FleetMenuItem()


@hooks.register("url_hook")
def register_urls():
    return UrlHook(urls, "fleet", "^fleet/")



from django.shortcuts import render, redirect
from django.db.models import Q
from django.contrib.auth.decorators import login_required, permission_required
from django.template.defaulttags import register

from fleet.app_settings import avoid_cdn

from .tasks import (
    open_fleet,
    send_fleet_invitation,
    get_fleet_composition,
    get_fleet_aggregate,
)

from .models import Fleet

from esi.decorators import token_required

from allianceauth.eveonline.models import EveCharacter
from allianceauth.groupmanagement.models import AuthGroup

from bravado.exception import HTTPNotFound


@login_required()
@permission_required("fleet.fleet_access")
def dashboard(request):
    groups = request.user.groups.all()
    fleets = Fleet.objects.filter(Q(groups__group__in=groups) | Q(groups=None)).all()
    context = {
        "fleets": fleets,
        "avoid_cdn": avoid_cdn(),  # AVOID_CDN setting
    }

    if "error_edit_fleet" in request.session:
        context["error"] = request.session["error_edit_fleet"].get("error", "")
        del request.session["error_edit_fleet"]

    return render(request, "fleet/dashboard.html", context)


@login_required()
@permission_required("fleet.manage")
@token_required(
    scopes=(
        "esi-fleets.read_fleet.v1",
        "esi-fleets.write_fleet.v1",
    )
)
def create_fleet(request, token):
    if request.method == "POST":
        auth_groups = AuthGroup.objects.filter(internal=False).all()
        if "modified_fleet_data" in request.session:
            ctx = {}
            ctx["error"] = request.session["modified_fleet_data"].get("error", "")
            ctx["motd"] = request.session["modified_fleet_data"].get("motd", "")
            ctx["name"] = request.session["modified_fleet_data"].get("name", "")
            ctx["groups"] = request.session["modified_fleet_data"].get("groups", "")
            ctx["is_free_move"] = request.session["modified_fleet_data"].get(
                "free_move", ""
            )
            ctx["character_id"] = token.character_id
            ctx["auth_groups"] = auth_groups

            del request.session["modified_fleet_data"]
        else:
            ctx = {"character_id": token.character_id, "auth_groups": auth_groups}

        # AVOID_CDN setting
        ctx["avoid_cdn"] = avoid_cdn()

        return render(request, "fleet/create_fleet.html", context=ctx)
    return redirect("fleet:dashboard")


@login_required()
@permission_required("fleet.manage")
def edit_fleet(request, fleet_id):
    fleet = Fleet.objects.get(fleet_id=fleet_id)
    auth_groups = AuthGroup.objects.filter(internal=False)

    ctx = {
        "character_id": fleet.fleet_commander_id,
        "auth_groups": auth_groups,
        "fleet": fleet,
        "avoid_cdn": avoid_cdn(),  # AVOID_CDN setting
    }

    return render(request, "fleet/edit_fleet.html", context=ctx)


@login_required()
@permission_required("fleet.fleet_access")
def join_fleet(request, fleet_id):
    ctx = {}
    groups = request.user.groups.all()
    fleet = Fleet.objects.filter(
        Q(groups__group__in=groups) | Q(groups=None), fleet_id=fleet_id
    ).count()

    if fleet == 0:
        return redirect("fleet:dashboard")

    if request.method == "POST":
        character_ids = request.POST.getlist("character_ids", [])
        send_fleet_invitation.delay(character_ids, fleet_id)

        return redirect("fleet:dashboard")
    else:
        characters = (
            EveCharacter.objects.filter(character_ownership__user=request.user)
            .select_related()
            .order_by("character_name")
        )
        ctx["characters"] = characters

    # AVOID_CDN setting
    ctx["avoid_cdn"] = avoid_cdn()

    return render(request, "fleet/join_fleet.html", context=ctx)


@login_required()
@permission_required("fleet.manage")
def save_fleet(request):
    if request.method == "POST":
        free_move = request.POST.get("free_move", False)

        if free_move == "on":
            free_move = True

        motd = request.POST.get("motd", "")
        name = request.POST.get("name", "")
        groups = request.POST.getlist("groups", [])

        try:
            open_fleet(request.POST["character_id"], motd, free_move, name, groups)
        except HTTPNotFound as ex:
            if request.POST.get("origin", "") == "edit":
                # Here ccp return character not in fleet. Instead push our own message to be clearer
                request.session["error_edit_fleet"] = {
                    "error": "Fleet advert is no longer valid"
                }

                return redirect("fleet:dashboard")

            if request.POST.get("origin", "") == "create":
                request.session["modified_fleet_data"] = {
                    "error": ex.swagger_result["error"],
                    "motd": motd,
                    "name": name,
                    "free_move": free_move,
                    "groups": groups,
                }

                return redirect("fleet:create_fleet")

    return redirect("fleet:dashboard")


@login_required()
@permission_required("fleet.manage")
def fleet_details(request, fleet_id):
    fleet = get_fleet_composition(fleet_id)

    ctx = {
        "fleet": fleet,
        "avoid_cdn": avoid_cdn(),  # AVOID_CDN setting
    }

    return render(request, "fleet/fleet_details.html", context=ctx)


@register.filter
def get_item(dictionary, key):
    return dictionary.get(key)



# Generated by Django 2.2.12 on 2020-08-16 19:07

import datetime
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("fleet", "0003_fleet_groups"),
    ]

    operations = [
        migrations.CreateModel(
            name="FleetInformation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("ship_type_name", models.CharField(max_length=100)),
                ("count", models.IntegerField()),
                ("date", models.DateTimeField(default=datetime.datetime.now)),
                (
                    "fleet",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="fleet.Fleet"
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
    ]



# Generated by Django 2.2.12 on 2020-06-13 00:17

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Fleet",
            fields=[
                ("fleet_id", models.BigIntegerField(primary_key=True, serialize=False)),
                ("fleet_commander_id", models.BigIntegerField()),
                ("created_at", models.DateTimeField()),
                ("motd", models.CharField(max_length=4000)),
                ("is_free_move", models.BooleanField()),
            ],
            options={
                "permissions": (
                    ("fleet_access", "Can access this app"),
                    ("manage", "Can Manage Fleets"),
                ),
                "default_permissions": (),
            },
        ),
    ]
